import { WebSocket } from "ws";
import { User } from "./user.js";
import { Beatmap } from "./beatmap.js";
import { Beatmapset } from "./beatmapset.js";
import { Multiplayer } from "./multiplayer.js";
import { Score } from "./score.js";
import { Ranking } from "./ranking.js";
import { Event } from "./event.js";
import { Changelog } from "./changelog.js";
import { Forum } from "./forum.js";
import { WikiPage } from "./wiki.js";
import { NewsPost } from "./news.js";
import { Home } from "./home.js";
import { Scope, Spotlight } from "./misc.js";
import { Chat } from "./chat.js";
import { Comment } from "./comment.js";
export { User } from "./user.js";
export { Beatmap } from "./beatmap.js";
export { Beatmapset } from "./beatmapset.js";
export { Multiplayer } from "./multiplayer.js";
export { Score } from "./score.js";
export { Ranking } from "./ranking.js";
export { Event } from "./event.js";
export { Changelog } from "./changelog.js";
export { Forum } from "./forum.js";
export { WikiPage } from "./wiki.js";
export { NewsPost } from "./news.js";
export { Home } from "./home.js";
export { Ruleset, Mod, Scope, Spotlight } from "./misc.js";
export { Chat } from "./chat.js";
export { WebSocket } from "./websocket.js";
export { Comment } from "./comment.js";
/**
 * Generates a link for users to click on in order to use your application!
 * @param client_id The Client ID, find it at https://osu.ppy.sh/home/account/edit#new-oauth-application
 * @param redirect_uri The specified Application Callback URL, aka where the user will be redirected upon clicking the button to authorize
 * @param scopes What you want to do with/as the user
 * @param server The API server (defaults to **https://osu.ppy.sh**, leave as is if you don't know exactly what you're doing)
 * @returns The link people should click on
 */
export declare function generateAuthorizationURL(client_id: number, redirect_uri: string, scopes: Scope[], server?: string): string;
/** If the `API` throws an error, it should always be an `APIError`! */
export declare class APIError {
    message: string;
    server: string;
    endpoint: string;
    parameters?: object;
    /**
     * @param message The reason why things didn't go as expected
     * @param server The server to which the request was sent
     * @param endpoint The type of resource that was requested from the server
     * @param parameters The filters that were used to specify what resource was wanted
     */
    constructor(message: string, server: string, endpoint: string, parameters?: object);
}
/** You can create an API instance without directly providing an access_token by using the API's `createAsync` function! {@link API.createAsync} */
export declare class API {
    access_token: string;
    client: {
        id: number;
        secret: string;
    };
    /** Should always be "Bearer" */
    token_type: string;
    /** The expiration date of your access_token */
    expires: Date;
    /** Which events should be logged (defaults to **none**) */
    verbose: "none" | "errors" | "all";
    /**
     * The base url of the server where the requests should land (defaults to **https://osu.ppy.sh**)
     * @remarks For tokens, requests will be sent to the `oauth/token` route, other requests will be sent to the `api/v2` route
     */
    server: string;
    /** If true, upon failing a request due to a 401, it will use the `refresh_token` and retry the request */
    refresh_on_401: boolean;
    /** If true, the application will silently use the `refresh_token` right before the `access_token` expires, as determined by `expires` */
    refresh_on_expires: boolean;
    /**
     * Valid for an unknown amount of time, allows you to get a new token without going through the Authorization Code Grant!
     * Use the API's `refreshToken` function to do that
     */
    refresh_token?: string;
    /** The osu! user id of the user who went through the Authorization Code Grant */
    user?: User["id"];
    /** The scopes your application have, assuming it acts as a user */
    scopes?: Scope[];
    /**
     * **Please use the API's `createAsync` method instead of the default constructor** if you don't have at least an `access_token`!
     * An API object without an `access_token` is pretty much useless!
     */
    constructor({ access_token, token_type, refresh_token, expires, scopes, user, server, client, verbose, refresh_on_401, refresh_on_expires }: {
        /** The token used in basically all requests! */
        access_token?: string;
        /** Should always be "Bearer" */
        token_type?: string;
        /** The token used to update your access_token and your refresh_token */
        refresh_token?: string;
        /** The expiration date of your access_token (doesn't affect application behaviour) */
        expires?: Date;
        /** The scopes your application have, assuming it acts as a user (doesn't affect application behaviour) */
        scopes?: Scope[];
        /** The id of the user this application acts as, if any (doesn't affect application behaviour) */
        user?: User["id"];
        /** The URL of the API server the package contacts */
        server?: string;
        /** The details of your application client, necessary for using the refresh_token */
        client?: {
            id: number;
            secret: string;
        };
        /** How much stuff should the package log */
        verbose?: "none" | "errors" | "all";
        /** If a 401 error is gotten from the server while it has a refresh token, should it use it and try the request again? (defaults to **true**) */
        refresh_on_401?: boolean;
        /** Should the application schedule a task to silently refresh the token right before the access_token expires? (defaults to **true**) */
        refresh_on_expires?: boolean;
    });
    /**
     * The normal way to create an API instance! Make sure to `await` it
     * @param client The ID and the secret of your client, can be found on https://osu.ppy.sh/home/account/edit#new-oauth-application
     * @param user If the instance is supposed to represent a user, use their Authorization Code and the Application Callback URL of your application!
     * @returns A promise with an API instance
     */
    static createAsync(client: {
        id: number;
        secret: string;
    }, user?: {
        /** The Application Callback URL; Where the User has been redirected to after saying "okay" to your application doing stuff */
        redirect_uri: string;
        /** The code that appeared as a GET argument when they got redirected to the Application Callback URl (`redirect_url`) */
        code: string;
    }, verbose?: "none" | "errors" | "all", server?: string): Promise<API>;
    /**
     * Use this instead of `console.log` to log any information
     * @param is_error Is the logging happening because of an error?
     * @param to_log Whatever you would put between the parentheses of `console.log()`
     */
    private log;
    /**
     * Get a websocket to get WebSocket events from!
     * @param server Where the "notification websocket/server" is
     * (defaults to **the api's `server`'s protocol and a maximum of 1 subdomain being replaced by "wss://notify."** (so usually `wss://notify.ppy.sh`))
    */
    generateWebSocket(server?: string): WebSocket;
    /**
     * Set most of an `api`'s properties, like tokens, token_type, scopes, expiration_date
     * @param body An Object with the client id & secret, grant_type, and stuff that depends of the grant_type
     * @param api The `api` which will see its properties change
     * @returns `api`, just in case, because in theory it should modify the original object
     */
    private getAndSetToken;
    /** @returns Whether or not the token has been refreshed */
    refreshToken(): Promise<boolean>;
    /** Revoke your current token! Revokes the refresh token as well */
    revokeToken(): Promise<true>;
    /**
     * The function that directly communicates with the API! Almost every functions of the API object uses this function!
     * @param method The type of request, each endpoint uses a specific one (if it uses multiple, the intent and parameters become different)
     * @param endpoint What comes in the URL after `api/`
     * @param parameters The things to specify in the request, such as the beatmap_id when looking for a beatmap
     * @param info Context given by a prior request
     * @returns A Promise with the API's response
     */
    request(method: "get" | "post" | "put" | "delete", endpoint: string, parameters?: {
        [k: string]: any;
    }, info?: {
        number_try: number;
        just_refreshed: boolean;
    }): Promise<any>;
    /** {@inheritDoc Beatmap.lookup} @group Beatmap Functions */
    readonly lookupBeatmap: typeof Beatmap.lookup;
    /** {@inheritDoc Beatmap.getOne} @group Beatmap Functions */
    readonly getBeatmap: typeof Beatmap.getOne;
    /** {@inheritDoc Beatmap.getMultiple} @group Beatmap Functions */
    readonly getBeatmaps: typeof Beatmap.getMultiple;
    /** {@inheritDoc Beatmap.DifficultyAttributes.get} @group Beatmap Functions */
    readonly getBeatmapDifficultyAttributes: typeof Beatmap.DifficultyAttributes.get;
    /** {@inheritDoc Beatmap.DifficultyAttributes.getOsu} @group Beatmap Functions */
    readonly getBeatmapDifficultyAttributesOsu: typeof Beatmap.DifficultyAttributes.getOsu;
    /** {@inheritDoc Beatmap.DifficultyAttributes.getTaiko} @group Beatmap Functions */
    readonly getBeatmapDifficultyAttributesTaiko: typeof Beatmap.DifficultyAttributes.getTaiko;
    /** {@inheritDoc Beatmap.DifficultyAttributes.getFruits} @group Beatmap Functions */
    readonly getBeatmapDifficultyAttributesFruits: typeof Beatmap.DifficultyAttributes.getFruits;
    /** {@inheritDoc Beatmap.DifficultyAttributes.getMania} @group Beatmap Functions */
    readonly getBeatmapDifficultyAttributesMania: typeof Beatmap.DifficultyAttributes.getMania;
    /** {@inheritDoc Beatmap.Pack.getOne} @group Beatmap Functions */
    readonly getBeatmapPack: typeof Beatmap.Pack.getOne;
    /** {@inheritDoc Beatmap.Pack.getMultiple} @group Beatmap Functions */
    readonly getBeatmapPacks: typeof Beatmap.Pack.getMultiple;
    /** {@inheritDoc Beatmap.getScores} @group Beatmap Functions */
    readonly getBeatmapScores: typeof Beatmap.getScores;
    /** {@inheritDoc Beatmap.getSoloScores} @group Beatmap Functions */
    readonly getBeatmapSoloScores: typeof Beatmap.getSoloScores;
    /** {@inheritDoc Beatmap.UserScore.getOne} @group Beatmap Functions */
    readonly getBeatmapUserScore: typeof Beatmap.UserScore.getOne;
    /** {@inheritDoc Beatmap.UserScore.getMultiple} @group Beatmap Functions */
    readonly getBeatmapUserScores: typeof Beatmap.UserScore.getMultiple;
    /** {@inheritDoc Beatmapset.search} @group Beatmapset Functions */
    readonly searchBeatmapsets: typeof Beatmapset.search;
    /** {@inheritDoc Beatmapset.lookup} @group Beatmapset Functions */
    readonly lookupBeatmapset: typeof Beatmapset.lookup;
    /** {@inheritDoc Beatmapset.getOne} @group Beatmapset Functions */
    readonly getBeatmapset: typeof Beatmapset.getOne;
    /** {@inheritDoc Beatmapset.Discussion.getMultiple} @group Beatmapset Functions */
    readonly getBeatmapsetDiscussions: typeof Beatmapset.Discussion.getMultiple;
    /** {@inheritDoc Beatmapset.Discussion.Post.getMultiple} @group Beatmapset Functions */
    readonly getBeatmapsetDiscussionPosts: typeof Beatmapset.Discussion.Post.getMultiple;
    /** {@inheritDoc Beatmapset.Discussion.Vote.getMultiple} @group Beatmapset Functions */
    readonly getBeatmapsetDiscussionVotes: typeof Beatmapset.Discussion.Vote.getMultiple;
    /** {@inheritDoc Beatmapset.Event.getMultiple} @group Beatmapset Functions */
    readonly getBeatmapsetEvents: typeof Beatmapset.Event.getMultiple;
    /** {@inheritDoc Changelog.Build.lookup} @group Changelog Functions */
    readonly lookupChangelogBuild: typeof Changelog.Build.lookup;
    /** {@inheritDoc Changelog.Build.getOne} @group Changelog Functions */
    readonly getChangelogBuild: typeof Changelog.Build.getOne;
    /** {@inheritDoc Changelog.Build.getMultiple} @group Changelog Functions */
    readonly getChangelogBuilds: typeof Changelog.Build.getMultiple;
    /** {@inheritDoc Changelog.UpdateStream.getAll} @group Changelog Functions */
    readonly getChangelogStreams: typeof Changelog.UpdateStream.getAll;
    /** {@inheritDoc Chat.keepAlive} @group Chat Functions */
    readonly keepChatAlive: typeof Chat.keepAlive;
    /** {@inheritDoc Chat.Message.getMultiple} @group Chat Functions */
    readonly getChatMessages: typeof Chat.Message.getMultiple;
    /** {@inheritDoc Chat.Message.send} @group Chat Functions */
    readonly sendChatMessage: typeof Chat.Message.send;
    /** {@inheritDoc Chat.Message.sendPrivate} @group Chat Functions */
    readonly sendChatPrivateMessage: typeof Chat.Message.sendPrivate;
    /** {@inheritDoc Chat.Channel.getOne} @group Chat Functions */
    readonly getChatChannel: typeof Chat.Channel.getOne;
    /** {@inheritDoc Chat.Channel.getAll} @group Chat Functions */
    readonly getChatChannels: typeof Chat.Channel.getAll;
    /** {@inheritDoc Chat.Channel.markAsRead} @group Chat Functions */
    readonly markChatChannelAsRead: typeof Chat.Channel.markAsRead;
    /** {@inheritDoc Chat.Channel.createPrivate} @group Chat Functions */
    readonly createChatPrivateChannel: typeof Chat.Channel.createPrivate;
    /** {@inheritDoc Chat.Channel.createAnnouncement} @group Chat Functions */
    readonly createChatAnnouncementChannel: typeof Chat.Channel.createAnnouncement;
    /** {@inheritDoc Chat.Channel.joinOne} @group Chat Functions */
    readonly joinChatChannel: typeof Chat.Channel.joinOne;
    /** {@inheritDoc Chat.Channel.leaveOne} @group Chat Functions */
    readonly leaveChatChannel: typeof Chat.Channel.leaveOne;
    /** {@inheritDoc Comment.getOne} @group Comment Functions */
    readonly getComment: typeof Comment.getOne;
    /** {@inheritDoc Comment.getMultiple} @group Comment Functions */
    readonly getComments: typeof Comment.getMultiple;
    /** {@inheritDoc Event.getMultiple} @group Event Functions */
    readonly getEvents: typeof Event.getMultiple;
    /** {@inheritDoc Forum.Topic.create} @group Forum Functions */
    readonly createForumTopic: typeof Forum.Topic.create;
    /** {@inheritDoc Forum.Topic.reply} @group Forum Functions */
    readonly replyForumTopic: typeof Forum.Topic.reply;
    /** {@inheritDoc Forum.Topic.editTitle} @group Forum Functions */
    readonly editForumTopicTitle: typeof Forum.Topic.editTitle;
    /** {@inheritDoc Forum.Post.edit} @group Forum Functions */
    readonly editForumPost: typeof Forum.Post.edit;
    /** {@inheritDoc Forum.getTopicAndPosts} @group Forum Functions */
    readonly getForumTopicAndPosts: typeof Forum.getTopicAndPosts;
    /** {@inheritDoc Home.Search.getUsers} @group Home Functions */
    readonly searchUser: typeof Home.Search.getUsers;
    /** {@inheritDoc Home.Search.getWikiPages} @group Home Functions */
    readonly searchWiki: typeof Home.Search.getWikiPages;
    /** {@inheritDoc Multiplayer.Room.getOne} @group Multiplayer Functions */
    readonly getRoom: typeof Multiplayer.Room.getOne;
    /** {@inheritDoc Multiplayer.Room.getOne} @group Multiplayer Functions */
    readonly getRooms: typeof Multiplayer.Room.getMultiple;
    /** {@inheritDoc Multiplayer.Room.getOne} @group Multiplayer Functions */
    readonly getRoomLeaderboard: typeof Multiplayer.Room.Leader.getMultiple;
    /** {@inheritDoc Multiplayer.Room.getOne} @group Multiplayer Functions */
    readonly getPlaylistItemScores: typeof Multiplayer.Room.PlaylistItem.getScores;
    /** {@inheritDoc Multiplayer.Room.getOne} @group Multiplayer Functions */
    readonly getMatch: typeof Multiplayer.Match.getOne;
    /** {@inheritDoc Multiplayer.Room.getOne} @group Multiplayer Functions */
    readonly getMatches: typeof Multiplayer.Match.getMultiple;
    /** {@inheritDoc NewsPost.getOne} @group NewsPost Functions */
    readonly getNewsPost: typeof NewsPost.getOne;
    /** {@inheritDoc NewsPost.getMultiple} @group NewsPost Functions */
    readonly getNewsPosts: typeof NewsPost.getMultiple;
    /** {@inheritDoc Ranking.getUser} @group Ranking Functions */
    readonly getUserRanking: typeof Ranking.getUser;
    /** {@inheritDoc Ranking.getCountry} @group Ranking Functions */
    readonly getCountryRanking: typeof Ranking.getCountry;
    /** {@inheritDoc Ranking.getKudosu} @group Ranking Functions */
    readonly getKudosuRanking: typeof Ranking.getKudosu;
    /** {@inheritDoc Ranking.getSpotlight} @group Ranking Functions */
    readonly getSpotlightRanking: typeof Ranking.getSpotlight;
    /** {@inheritDoc User.getResourceOwner} @group User Functions */
    readonly getResourceOwner: typeof User.getResourceOwner;
    /** {@inheritDoc User.getOne} @group User Functions */
    readonly getUser: typeof User.getOne;
    /** {@inheritDoc User.getMultiple} @group User Functions */
    readonly getUsers: typeof User.getMultiple;
    /** {@inheritDoc User.getScores} @group User Functions */
    readonly getUserScores: typeof User.getScores;
    /** {@inheritDoc User.getBeatmaps} @group User Functions */
    readonly getUserBeatmaps: typeof User.getBeatmaps;
    /** {@inheritDoc User.getMostPlayed} @group User Functions */
    readonly getUserMostPlayed: typeof User.getMostPlayed;
    /** {@inheritDoc User.getRecentActivity} @group User Functions */
    readonly getUserRecentActivity: typeof User.getRecentActivity;
    /** {@inheritDoc User.getKudosu} @group User Functions */
    readonly getUserKudosu: typeof User.getKudosu;
    /** {@inheritDoc User.getFriends} @group User Functions */
    readonly getFriends: typeof User.getFriends;
    /** {@inheritDoc WikiPage.getOne} @group WikiPage Functions */
    readonly getWikiPage: typeof WikiPage.getOne;
    /** {@inheritDoc Spotlight.getAll} @group Other Functions */
    readonly getSpotlights: typeof Spotlight.getAll;
    /** {@inheritDoc Score.getReplay} @group Other Functions */
    readonly getReplay: typeof Score.getReplay;
    /**
     * Get the backgrounds made and selected for this season or for last season!
     * @returns When the season ended, and for each background, its URL and its artist
     * @group Other Functions
     */
    getSeasonalBackgrounds(): Promise<{
        ends_at: Date;
        backgrounds: {
            url: string;
            user: User;
        }[];
    }>;
}
